/*
 * $Id: PayloadInvocablePojoWrapper.java 2443 2009-09-23 10:05:21Z magicduncan $
 * $URL: https://open.jira.com/svn/IKASAN/tags/ikasan-0.8.4/framework/src/main/java/org/ikasan/framework/plugins/PayloadInvocablePojoWrapper.java $
 * 
 * ====================================================================
 * Ikasan Enterprise Integration Platform
 * 
 * Distributed under the Modified BSD License.
 * Copyright notice: The copyright for this software and a full listing 
 * of individual contributors are as shown in the packaged copyright.txt 
 * file. 
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the ORGANIZATION nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.ikasan.framework.plugins;

import org.ikasan.common.Payload;
import org.ikasan.framework.plugins.invoker.PluginInvocationException;

/**
 * Wrapper class for plugins that do not implement the <code>PayloadInvocable</code> interface, but nonetheless have a
 * functional method that matches the parameter list.
 * 
 * All wrapped plugins must still provide a public void method that takes as arguments a <code>Payload</code>, along
 * with a <code>TargetParams</code> object
 * 
 * @author Ikasan Development Team
 */
public class PayloadInvocablePojoWrapper extends PojoWrapperPlugin implements PayloadInvocable
{
    /**
     * Constructor
     * 
     * @param pojo POJO to wrap
     * @param pojoMethodName Method on the POJO to execute
     */
    public PayloadInvocablePojoWrapper(Object pojo, String pojoMethodName)
    {
        super(pojo, pojoMethodName);
    }

    public void invoke(Payload payload) throws PluginInvocationException
    {
        invoke(new Object[] { payload }, new Class[] { Payload.class });
    }
}
